/*
----------------------------------------------------------------------------
----                                                                    ----
----  This file is part of the WATCHDOG project                         ----
----                                                                    ----
----  Description:                                                      ----
----  Implementation of a WATCHDOG for RISCMCU                          ----
----                                                                    ----
----                                                                    ----
----------------------------------------------------------------------------
----                                                                    ----
----  Copyright (C) 2006 P. Sarro, C. Sasso, G. Scappatura              ----
----                                                                    ----
----  This source file may be used and distributed without              ----
----  restriction provided that this copyright statement is not         ----
----  removed from the file and that any derivative work contains       ----
----  the original copyright notice and the associated disclaimer.      ----
----                                                                    ----
----  This source file is free software; you can redistribute it        ----
----  and/or modify it under the terms of the GNU Lesser General        ----
----  Public License as published by the Free Software Foundation;      ----
----  either version 2.1 of the License, or (at your option) any        ----
----  later version.                                                    ----
----                                                                    ----
----  This source is distributed in the hope that it will be            ----
----  useful, but WITHOUT ANY WARRANTY; without even the implied        ----
----  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           ----
----  PURPOSE. See the GNU Lesser General Public License for more       ----
----  details.                                                          ----
----                                                                    ----
----  A copy of the license can be found at                             ----
----  http://www.fsf.org/licensing/licenses/lgpl.html                   ----
----                                                                    ----
----------------------------------------------------------------------------
*/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#define MAX_CHARS 256
#define MAX_FILESIZE 102400 //=100KB

void usage()
{
    printf("RomGen: generate vhdl files for the watchdog.\n\n");
    printf("usage: RomGen inputfile\n\n");
    printf("\tinputfile\tsignal fingerprint file.\n\n");
}

int readline(FILE *file, char *string)
{
    char ch;
    int i;

    ch=fgetc(file);
    for(i=0;(ch!='\n' && ch!=EOF);i++)
    {
        string[i]=ch;
        ch=fgetc(file);
    }
    string[i]='\0';

    if (ch==EOF && i==0)
        return EOF;
    else
        return i;
}

int main(int argc, char *argv[])
{
    char inputfilename[MAX_CHARS], line[MAX_CHARS], instruction[MAX_CHARS], signal[MAX_CHARS], signalmask[MAX_CHARS], vhdltext[MAX_FILESIZE], data[MAX_FILESIZE], rom_sf_data[MAX_FILESIZE], rom_mf_data[MAX_FILESIZE], rom_se_data[MAX_FILESIZE], rom_me_data[MAX_FILESIZE];
    FILE *inputfile, *templatefile, *rom_sf_file, *rom_mf_file, *rom_se_file, *rom_me_file;
    int i, ncelle, nbitin, nbitout;
    char *chrptr;

    if (argc<2)
    {
        usage();
        exit(1);
    }
    else
        strcpy(inputfilename,argv[1]);

    inputfile=fopen(inputfilename,"r");
    if (inputfile==NULL)
    {
        printf("Unable to access the specified inputfile.\nPlease check name and path, then try again.\nProgram terminated.\n");
        exit(1);
    }

    templatefile=fopen("rom_template.vhd","r");
    i=0;
    while ((vhdltext[i]=fgetc(templatefile))!=EOF)
    {
        i++;
    }
    vhdltext[i]='\0';

    rom_sf_file=fopen("rom_sf.vhd","w");
    rom_mf_file=fopen("rom_mf.vhd","w");
    rom_se_file=fopen("rom_se.vhd","w");
    rom_me_file=fopen("rom_me.vhd","w");

    rom_sf_data[0]='\0';
    rom_mf_data[0]='\0';
    rom_se_data[0]='\0';
    rom_me_data[0]='\0';

    ncelle=0;
    while (readline(inputfile,line)!=EOF)
    {
        //printf("letto: %s",line);
        if (line[0]=='#')
        {
            strcpy(instruction,line+1);
            ncelle++;
        }
        else if (line[0]=='F' || line[0]=='E')
        {
            strcpy(signal,line+2);
            for (i=0;signal[i]!='\0';i++)
            {
                signalmask[i]='1';
                if (signal[i]=='-')
                {
                    signal[i]='0';
                    signalmask[i]='0';
                }

            }
            signalmask[i]='\0';

            if (line[0]=='F')
            {
                sprintf(data,"                                \"%s\", --%s\n",signal,instruction);
                strcat(rom_sf_data,data);
                sprintf(data,"                                \"%s\", --%s\n",signalmask,instruction);
                strcat(rom_mf_data,data);
            }
            else  if (line[0]=='E')
            {
                sprintf(data,"                                \"%s\", --%s\n",signal,instruction);
                strcat(rom_se_data,data);
                sprintf(data,"                                \"%s\", --%s\n",signalmask,instruction);
                strcat(rom_me_data,data);
            }
        }
    }
    /*
    strcpy(rom_sf_data,"\"000011110000111100001111\"");
    strcpy(rom_mf_data,"\"000011110000111100001111\"");
    strcpy(rom_se_data,"\"000011110000111100001111\"");
    strcpy(rom_me_data,"\"000011110000111100001111\"");
    */
    chrptr=strrchr(rom_sf_data,',');
    *chrptr=' ';
    chrptr=strrchr(rom_mf_data,',');
    *chrptr=' ';
    chrptr=strrchr(rom_se_data,',');
    *chrptr=' ';
    chrptr=strrchr(rom_me_data,',');
    *chrptr=' ';

    nbitout=strlen(signal);
    nbitin=ceil(log10(ncelle)/log10(2));

    fprintf(rom_sf_file,vhdltext,"rom_sf","ncelle",ncelle,"nbitout",nbitout,"nbitin",nbitin,"rom_sf","rom_sf",rom_sf_data);
    fprintf(rom_mf_file,vhdltext,"rom_mf","ncelle",ncelle,"nbitout",nbitout,"nbitin",nbitin,"rom_mf","rom_mf",rom_mf_data);
    fprintf(rom_se_file,vhdltext,"rom_se","ncelle",ncelle,"nbitout",nbitout,"nbitin",nbitin,"rom_se","rom_se",rom_se_data);
    fprintf(rom_me_file,vhdltext,"rom_me","ncelle",ncelle,"nbitout",nbitout,"nbitin",nbitin,"rom_me","rom_me",rom_me_data);

    fclose(inputfile);
    fclose(templatefile);
    fclose(rom_sf_file);
    fclose(rom_mf_file);
    fclose(rom_se_file);
    fclose(rom_me_file);

    printf("rom files generated succesfully.\n");
    return 0;
}
