----------------------------------------------------------------------------
----                                                                    ----
----  This file is part of the WATCHDOG project                         ----
----                                                                    ----
----  Description:                                                      ----
----  Implementation of a WATCHDOG for RISCMCU                          ----
----                                                                    ----
----                                                                    ----
----------------------------------------------------------------------------
----                                                                    ----
----  Copyright (C) 2006 P. Sarro, C. Sasso, G. Scappatura              ----
----                                                                    ----
----  This source file may be used and distributed without              ----
----  restriction provided that this copyright statement is not         ----
----  removed from the file and that any derivative work contains       ----
----  the original copyright notice and the associated disclaimer.      ----
----                                                                    ----
----  This source file is free software; you can redistribute it        ----
----  and/or modify it under the terms of the GNU Lesser General        ----
----  Public License as published by the Free Software Foundation;      ----
----  either version 2.1 of the License, or (at your option) any        ----
----  later version.                                                    ----
----                                                                    ----
----  This source is distributed in the hope that it will be            ----
----  useful, but WITHOUT ANY WARRANTY; without even the implied        ----
----  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           ----
----  PURPOSE. See the GNU Lesser General Public License for more       ----
----  details.                                                          ----
----                                                                    ----
----  A copy of the license can be found at                             ----
----  http://www.fsf.org/licensing/licenses/lgpl.html                   ----
----                                                                    ----
----------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY comparator_tb IS
END comparator_tb;

ARCHITECTURE behavior OF comparator_tb IS 

	COMPONENT comparator
    generic (
      nbit : integer);
	PORT(
		inA : IN std_logic_vector(nbit-1 downto 0);
		inB : IN std_logic_vector(nbit-1 downto 0);          
		out0 : OUT std_logic
		);
	END COMPONENT;

  constant nbit : integer := 4;

	SIGNAL out0 :  std_logic;
	SIGNAL inA :  std_logic_vector(nbit-1 downto 0);
	SIGNAL inB :  std_logic_vector(nbit-1 downto 0);

  constant clk_period : time := 10 ns;

BEGIN

	DUT: comparator
    generic map (
        nbit => nbit)
	PORT MAP(
		out0 => out0,
		inA => inA,
		inB => inB
	);


  inA <= "0101", "0111" after 7*clk_period, "0011" after 12*clk_period, "0000" after 15*clk_period;
  inB <= "0100", "0111" after 7*clk_period, "0011" after 10*clk_period, "0000" after 13*clk_period;

END;
