----------------------------------------------------------------------------
----                                                                    ----
----  This file is part of the WATCHDOG project                         ----
----                                                                    ----
----  Description:                                                      ----
----  Implementation of a WATCHDOG for RISCMCU                          ----
----                                                                    ----
----                                                                    ----
----------------------------------------------------------------------------
----                                                                    ----
----  Copyright (C) 2006 P. Sarro, C. Sasso, G. Scappatura              ----
----                                                                    ----
----  This source file may be used and distributed without              ----
----  restriction provided that this copyright statement is not         ----
----  removed from the file and that any derivative work contains       ----
----  the original copyright notice and the associated disclaimer.      ----
----                                                                    ----
----  This source file is free software; you can redistribute it        ----
----  and/or modify it under the terms of the GNU Lesser General        ----
----  Public License as published by the Free Software Foundation;      ----
----  either version 2.1 of the License, or (at your option) any        ----
----  later version.                                                    ----
----                                                                    ----
----  This source is distributed in the hope that it will be            ----
----  useful, but WITHOUT ANY WARRANTY; without even the implied        ----
----  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           ----
----  PURPOSE. See the GNU Lesser General Public License for more       ----
----  details.                                                          ----
----                                                                    ----
----  A copy of the license can be found at                             ----
----  http://www.fsf.org/licensing/licenses/lgpl.html                   ----
----                                                                    ----
----------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity encoder_16_6 is
  generic (  nbitin  :     integer := 16;  
            nbitout :     integer := 6);  
    Port (
    		 in0 : in std_logic_vector(nbitin-1 downto 0);
           out0 : out std_logic_vector(nbitout-1 downto 0));
end encoder_16_6;

architecture dataflow1 of encoder_16_6 is

	constant delay : time := 10.337 ns;

begin
   out0 <= conv_std_logic_vector(0,nbitout) after delay when ( in0="0000000000000000" ) else	   -- NOP		--vedi che si pu accorciare
	   conv_std_logic_vector(1,nbitout) after delay when ( in0(15 downto 10)="000001" ) else -- CPC
	   conv_std_logic_vector(2,nbitout) after delay when ( in0(15 downto 10)="000010" ) else -- SBC
	   conv_std_logic_vector(3,nbitout) after delay when ( in0(15 downto 10)="000011" ) else -- ADD (LSL)
	   conv_std_logic_vector(4,nbitout) after delay when ( in0(15 downto 10)="000100" ) else -- CPSE
	   conv_std_logic_vector(5,nbitout) after delay when ( in0(15 downto 10)="000101" ) else -- CP
	   conv_std_logic_vector(6,nbitout) after delay when ( in0(15 downto 10)="000110" ) else -- SUB
	   conv_std_logic_vector(7,nbitout) after delay when ( in0(15 downto 10)="000111" ) else -- ADC (ROL)
	   conv_std_logic_vector(8,nbitout) after delay when ( in0(15 downto 10)="001000" ) else -- AND (TST)
	   conv_std_logic_vector(9,nbitout) after delay when ( in0(15 downto 10)="001001" ) else -- EOR (CLR)
	   conv_std_logic_vector(10,nbitout) after delay when ( in0(15 downto 10)="001010" ) else -- OR 
	   conv_std_logic_vector(11,nbitout) after delay when ( in0(15 downto 10)="001011" ) else -- MOV 
	   conv_std_logic_vector(12,nbitout) after delay when ( in0(15 downto 12)="0011" ) else -- CPI
	   conv_std_logic_vector(13,nbitout) after delay when ( in0(15 downto 12)="0100" ) else -- SBCI
	   conv_std_logic_vector(14,nbitout) after delay when ( in0(15 downto 12)="0101" ) else -- SUBI
	   conv_std_logic_vector(15,nbitout) after delay when ( in0(15 downto 12)="0110" ) else -- ORI
	   conv_std_logic_vector(16,nbitout) after delay when ( in0(15 downto 12)="0111" ) else -- ANDI
	   conv_std_logic_vector(17,nbitout) after delay when ( in0(15 downto 9)="1000000" and in0(3 downto 0)="0000") else -- LD --vedi che si pu togliere la and
	   conv_std_logic_vector(18,nbitout) after delay when ( in0(15 downto 9)="1000001" and in0(3 downto 0)="0000") else -- ST --vedi che si pu togliere la and
	   conv_std_logic_vector(19,nbitout) after delay when ( in0(15 downto 9)="1001000" and in0(3 downto 0)="0001") else -- LD Z+
	   conv_std_logic_vector(20,nbitout) after delay when ( in0(15 downto 9)="1001000" and in0(3 downto 0)="0010") else -- LD -Z
	   conv_std_logic_vector(21,nbitout) after delay when ( in0(15 downto 9)="1001001" and in0(3 downto 0)="0001") else -- ST Z+
	   conv_std_logic_vector(22,nbitout) after delay when ( in0(15 downto 9)="1001001" and in0(3 downto 0)="0010") else -- ST -Z
	   conv_std_logic_vector(23,nbitout) after delay when ( in0(15 downto 9)="1001010" and in0(3 downto 0)="0000") else -- COM
	   conv_std_logic_vector(24,nbitout) after delay when ( in0(15 downto 9)="1001010" and in0(3 downto 0)="0001") else -- NEG
	   conv_std_logic_vector(25,nbitout) after delay when ( in0(15 downto 9)="1001010" and in0(3 downto 0)="0010") else -- SWAP
	   conv_std_logic_vector(26,nbitout) after delay when ( in0(15 downto 9)="1001010" and in0(3 downto 0)="0011") else -- INC
	   conv_std_logic_vector(27,nbitout) after delay when ( in0(15 downto 9)="1001010" and in0(3 downto 0)="0101") else -- ASR
	   conv_std_logic_vector(28,nbitout) after delay when ( in0(15 downto 9)="1001010" and in0(3 downto 0)="0110") else -- LSR
	   conv_std_logic_vector(29,nbitout) after delay when ( in0(15 downto 9)="1001010" and in0(3 downto 0)="0111") else -- ROR
	   conv_std_logic_vector(30,nbitout) after delay when ( in0(15 downto 9)="1001010" and in0(3 downto 0)="1010") else -- DEC
	   conv_std_logic_vector(31,nbitout) after delay when ( in0(15 downto 7)="100101000" and in0(3 downto 0)="1000") else -- BSET (SE? - I,T,H,S,V,N,Z,C)
	   conv_std_logic_vector(32,nbitout) after delay when ( in0(15 downto 7)="100101001" and in0(3 downto 0)="1000") else -- BCLR (CL? - I,T,H,S,V,N,Z,C)
	   conv_std_logic_vector(33,nbitout) after delay when ( in0="1001010100001000" ) else	   -- RET
	   conv_std_logic_vector(34,nbitout) after delay when ( in0="1001010100011000" ) else	   -- RETI
	   conv_std_logic_vector(35,nbitout) after delay when ( in0="1001010110001000" ) else	   -- SLEEP
	   conv_std_logic_vector(36,nbitout) after delay when ( in0(15 downto 8)="10011000" ) else -- CBI
	   conv_std_logic_vector(37,nbitout) after delay when ( in0(15 downto 8)="10011001" ) else -- SBIC
	   conv_std_logic_vector(38,nbitout) after delay when ( in0(15 downto 8)="10011010" ) else -- SBI
	   conv_std_logic_vector(39,nbitout) after delay when ( in0(15 downto 8)="10011011" ) else -- SBIS
	   conv_std_logic_vector(40,nbitout) after delay when ( in0(15 downto 11)="10110" ) else -- IN
	   conv_std_logic_vector(41,nbitout) after delay when ( in0(15 downto 11)="10111" ) else -- OUT
	   conv_std_logic_vector(42,nbitout) after delay when ( in0(15 downto 12)="1100" ) else -- RJMP
	   conv_std_logic_vector(43,nbitout) after delay when ( in0(15 downto 12)="1101" ) else -- RCALL
	   conv_std_logic_vector(44,nbitout) after delay when ( in0(15 downto 12)="1110" ) else -- LDI (SER)
	   conv_std_logic_vector(45,nbitout) after delay when ( in0(15 downto 10)="111100" ) else -- BRBS (BR?? - CS,LO,EQ,MI,VS,LT,HS,TS,IE)
	   conv_std_logic_vector(46,nbitout) after delay when ( in0(15 downto 10)="111101" ) else -- BRBC (BR?? - CC,SH,NE,PL,VC,GE,HC,TC,ID)
	   conv_std_logic_vector(47,nbitout) after delay when ( in0(15 downto 9)="1111100" ) else -- BLD 
	   conv_std_logic_vector(48,nbitout) after delay when ( in0(15 downto 9)="1111101" ) else -- BST 
	   conv_std_logic_vector(49,nbitout) after delay when ( in0(15 downto 9)="1111110" ) else -- SBRC 
	   conv_std_logic_vector(50,nbitout) after delay when ( in0(15 downto 9)="1111111" ) else -- SBRS
	   (others =>'0') after delay; 


end dataflow1;
