----------------------------------------------------------------------------
----                                                                    ----
----  This file is part of the WATCHDOG project                         ----
----                                                                    ----
----  Description:                                                      ----
----  Implementation of a WATCHDOG for RISCMCU                          ----
----                                                                    ----
----                                                                    ----
----------------------------------------------------------------------------
----                                                                    ----
----  Copyright (C) 2006 P. Sarro, C. Sasso, G. Scappatura              ----
----                                                                    ----
----  This source file may be used and distributed without              ----
----  restriction provided that this copyright statement is not         ----
----  removed from the file and that any derivative work contains       ----
----  the original copyright notice and the associated disclaimer.      ----
----                                                                    ----
----  This source file is free software; you can redistribute it        ----
----  and/or modify it under the terms of the GNU Lesser General        ----
----  Public License as published by the Free Software Foundation;      ----
----  either version 2.1 of the License, or (at your option) any        ----
----  later version.                                                    ----
----                                                                    ----
----  This source is distributed in the hope that it will be            ----
----  useful, but WITHOUT ANY WARRANTY; without even the implied        ----
----  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           ----
----  PURPOSE. See the GNU Lesser General Public License for more       ----
----  details.                                                          ----
----                                                                    ----
----  A copy of the license can be found at                             ----
----  http://www.fsf.org/licensing/licenses/lgpl.html                   ----
----                                                                    ----
----------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY ff_d_tb IS
END ff_d_tb;

ARCHITECTURE behavior OF ff_d_tb IS 

	COMPONENT ff_d
	PORT(
		D : IN std_logic;
		EN : in std_logic;
		CLK : IN std_logic;
		RSTn : IN std_logic;          
		Q : OUT std_logic
		);
	END COMPONENT;

	SIGNAL D :  std_logic;
	SIGNAL EN : std_logic;
	SIGNAL CLK :  std_logic:='0';
	SIGNAL RSTn :  std_logic:='1';
	SIGNAL Q :  std_logic;

	constant clk_period : time := 10 ns;

BEGIN

	DUT: ff_d PORT MAP(
		D => D,
		EN => EN,
		CLK => CLK,
		RSTn => RSTn,
		Q => Q
	);

  rstn <= '0'      after 1 ns, '1' after 2 ns, '0' after 8*clk_period, '1' after 9*clk_period;
  clk <= not(clk) after clk_period/2;
  D <= '0', '1' after 2*clk_period, '0' after 5*clk_period, '1' after 6*clk_period, '1' after 7*clk_period;
  EN <= '1', '0' after 6*clk_period, '1' after 7*clk_period;

END;
