----------------------------------------------------------------------------
----                                                                    ----
----  This file is part of the WATCHDOG project                         ----
----                                                                    ----
----  Description:                                                      ----
----  Implementation of a WATCHDOG for RISCMCU                          ----
----                                                                    ----
----                                                                    ----
----------------------------------------------------------------------------
----                                                                    ----
----  Copyright (C) 2006 P. Sarro, C. Sasso, G. Scappatura              ----
----                                                                    ----
----  This source file may be used and distributed without              ----
----  restriction provided that this copyright statement is not         ----
----  removed from the file and that any derivative work contains       ----
----  the original copyright notice and the associated disclaimer.      ----
----                                                                    ----
----  This source file is free software; you can redistribute it        ----
----  and/or modify it under the terms of the GNU Lesser General        ----
----  Public License as published by the Free Software Foundation;      ----
----  either version 2.1 of the License, or (at your option) any        ----
----  later version.                                                    ----
----                                                                    ----
----  This source is distributed in the hope that it will be            ----
----  useful, but WITHOUT ANY WARRANTY; without even the implied        ----
----  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           ----
----  PURPOSE. See the GNU Lesser General Public License for more       ----
----  details.                                                          ----
----                                                                    ----
----  A copy of the license can be found at                             ----
----  http://www.fsf.org/licensing/licenses/lgpl.html                   ----
----                                                                    ----
----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity pipo is
  generic (
    nbit     :     integer := 16);
  port (
    out0     : out std_logic_vector (nbit-1 downto 0);
    in0      : in  std_logic_vector (nbit-1 downto 0);
    enable   : in  std_logic;
    clk      : in  std_logic;
    rstn      : in  std_logic);
end pipo;

architecture behav1 of pipo is

	constant delay : time := 3.615 ns;

  signal mem0 : std_logic_vector (nbit-1 downto 0);

begin  -- behav1

  process (clk, rstn)

  begin  -- process
    if rstn = '0' then                   -- asynchronous reset (active high)
      mem0 <= (others => '0');

    elsif clk'event and clk = '1' then  -- rising clock edge
      if enable = '1' then
        mem0 <= in0;                    --carica
      end if;
    end if;

  end process;

  out0 <= mem0 after delay;

end behav1;
