----------------------------------------------------------------------------
----                                                                    ----
----  This file is part of the WATCHDOG project                         ----
----                                                                    ----
----  Description:                                                      ----
----  Implementation of a WATCHDOG for RISCMCU                          ----
----                                                                    ----
----                                                                    ----
----------------------------------------------------------------------------
----                                                                    ----
----  Copyright (C) 2006 P. Sarro, C. Sasso, G. Scappatura              ----
----                                                                    ----
----  This source file may be used and distributed without              ----
----  restriction provided that this copyright statement is not         ----
----  removed from the file and that any derivative work contains       ----
----  the original copyright notice and the associated disclaimer.      ----
----                                                                    ----
----  This source file is free software; you can redistribute it        ----
----  and/or modify it under the terms of the GNU Lesser General        ----
----  Public License as published by the Free Software Foundation;      ----
----  either version 2.1 of the License, or (at your option) any        ----
----  later version.                                                    ----
----                                                                    ----
----  This source is distributed in the hope that it will be            ----
----  useful, but WITHOUT ANY WARRANTY; without even the implied        ----
----  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           ----
----  PURPOSE. See the GNU Lesser General Public License for more       ----
----  details.                                                          ----
----                                                                    ----
----  A copy of the license can be found at                             ----
----  http://www.fsf.org/licensing/licenses/lgpl.html                   ----
----                                                                    ----
----------------------------------------------------------------------------
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

-------------------------------------------------------------------------------

entity pipo_tb is

end pipo_tb;

-------------------------------------------------------------------------------

architecture test of pipo_tb is

  component pipo
    generic (
      nbit : integer);
    port (
      out0     : out std_logic_vector (nbit-1 downto 0);
      in0      : in  std_logic_vector (nbit-1 downto 0);
      enable     : in  std_logic;
      clk      : in  std_logic;
      rstn      : in  std_logic);
  end component;

  constant nbit : integer := 13;

  signal out0     : std_logic_vector (nbit-1 downto 0);
  signal in0      : std_logic_vector (nbit-1 downto 0);
  signal enable     : std_logic;
  signal clk      : std_logic:='0';
  signal rstn      : std_logic:='1';
  
  constant clk_period : time := 10 ns;

begin  -- test

  DUT: pipo
    generic map (
        nbit => nbit)
    port map (
        out0     => out0,
        in0      => in0,
        enable => enable,
        clk      => clk,
        rstn      => rstn);

  rstn <= '0'      after 1 ns, '1' after 2 ns, '0' after 8*clk_period, '1' after 9*clk_period;
  clk <= not(clk) after clk_period/2;

  enable <= '0', '1' after clk_period, '0' after 2*clk_period, '1' after 8*clk_period, '0' after 9*clk_period, '1' after 11*clk_period, '0' after 12*clk_period;

  in0 <= "0101001001001", "0101001111001" after 7*clk_period, "0100001001001" after 12*clk_period;




end test;

-------------------------------------------------------------------------------
