----------------------------------------------------------------------------
----                                                                    ----
----  This file is part of the WATCHDOG project                         ----
----                                                                    ----
----  Description:                                                      ----
----  Implementation of a WATCHDOG for RISCMCU                          ----
----                                                                    ----
----                                                                    ----
----------------------------------------------------------------------------
----                                                                    ----
----  Copyright (C) 2006 P. Sarro, C. Sasso, G. Scappatura              ----
----                                                                    ----
----  This source file may be used and distributed without              ----
----  restriction provided that this copyright statement is not         ----
----  removed from the file and that any derivative work contains       ----
----  the original copyright notice and the associated disclaimer.      ----
----                                                                    ----
----  This source file is free software; you can redistribute it        ----
----  and/or modify it under the terms of the GNU Lesser General        ----
----  Public License as published by the Free Software Foundation;      ----
----  either version 2.1 of the License, or (at your option) any        ----
----  later version.                                                    ----
----                                                                    ----
----  This source is distributed in the hope that it will be            ----
----  useful, but WITHOUT ANY WARRANTY; without even the implied        ----
----  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           ----
----  PURPOSE. See the GNU Lesser General Public License for more       ----
----  details.                                                          ----
----                                                                    ----
----  A copy of the license can be found at                             ----
----  http://www.fsf.org/licensing/licenses/lgpl.html                   ----
----                                                                    ----
----------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity rom_mf is
  generic (
    ncelle  : integer := 51;
    nbitout  : integer := 24;
    nbitin  : integer := 6);
  port (
    in0  : in  std_logic_vector (nbitin-1 downto 0);
    out0 : out std_logic_vector (nbitout-1 downto 0));
end rom_mf;

architecture behav1 of rom_mf is

  type std_logic_matrix is array (integer range <>) of std_logic_vector (nbitout-1 downto 0);

	constant delay : time := 6.117 ns;

  constant mem0 : std_logic_matrix (0 to ncelle-1 ) := (
                                "001000000100000000100000", --NOP  (01)
                                "000000001000000000000000", --CPC  (02)
                                "000000000000000100000000", --SBC  (03)
                                "010000000000010000000000", --ADD  (04)
                                "000010100000000000000000", --CPSE (05)
                                "100000000000000100000000", --CP   (06)
                                "010000001010110000001010", --SUB  (07)
                                "000011000000000000000000", --ADC  (08)
                                "000000010110000000010000", --AND  (09)
                                "100000000000000000000000", --EOR  (10)
                                "100000000000000000000000", --OR   (11)
                                "000000000000000010010000", --MOV  (12)
                                "001000000000000100000000", --CPI  (13)
                                "000100000000000100000000", --SBCI (14)
                                "000000000000100000000000", --SUBI (15)
                                "011001000000001000000000", --ORI  (16)
                                "000000010000000000000001", --ANDI (17)
                                "000000000000001001001000", --LD   (18)
                                "110000001000000000100000", --ST   (19)
                                "100000000000000000000000", --LDZ+ (20)
                                "000000000000000001000000", --LDZ (21)
                                "000010100000000000000001", --STZ+ (22)
                                "100000000000000000010000", --STZ (23)
                                "000000000000000000110001", --COM  (24)
                                "000000110001100000000000", --NEG  (25)
                                "010000010000000000011000", --SWAP (26)
                                "000000001000100000000000", --INC  (27)
                                "000001000000001100000100", --ASR  (28)
                                "100000000000000000000000", --LSR  (29)
                                "000000000000100000000000", --ROR  (30)
                                "000000000100001000000000", --DEC  (31)
                                "000000100010000000100110", --BSET (32)
                                "100000000000000000000000", --BCLR (33)
                                "100000000000100000000000", --RET  (34)
                                "010010000000000000000011", --RETI (35)
                                "000010000000000000000000", --SLEEP(36)
                                "000000000000000000001100", --CBI  (37)
                                "000000110000100000000000", --SBIC (38)
                                "010000000000000000000100", --SBI  (39)
                                "000000100000000000000000", --SBIS (40)
                                "100000100000000010001000", --IN   (41)
                                "000100100000000000000000", --OUT  (42)
                                "000000100000000000000000", --RJMP (43)
                                "000000001000000000000000", --RCALL(44)
                                "000000000000000110000100", --LDI  (45)
                                "000100000000000000100000", --BRBS (46)
                                "000000000000001000000000", --BRBC (47)
                                "000000000000000001000000", --BLD  (48)
                                "001000000000001000000000", --BST  (49)
                                "000000000001001010000100", --SBRC (50)
                                "000000000000001000010100"  --SBRS (51)
                                                     );
  
begin

  out0 <= mem0(conv_integer(in0)) after delay;
  
end behav1;

